import django_tables2 as tables
from netbox.tables import NetBoxTable, ChoiceFieldColumn

from .models import gns3srv, ptovjob, switchtojob


#class ptovTable(NetBoxTable):
#    name = tables.Column(linkify=True)
#
#    class Meta(NetBoxTable.Meta):
#        model = ptov
#        fields = ("pk", "id", "name", "actions")
#        default_columns = ("name",)

class gns3srvTable(NetBoxTable):
    name = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = gns3srv 
        fields = ("pk", "id", "name", "actions")
        default_columns = ("name",)

class ptovjobTable(NetBoxTable):
    name = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = ptovjob
        fields = ("pk", "id", "name", "gns3srv", "gns3prjname", "gns3prjid", "eosuname", "eospasswd")
        default_columns = ("name",)

class switchtojobTable(NetBoxTable):
    name = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = switchtojob
        fields = ("pk", "id", 'name', "job", "switch")
        default_columns = ("name",)



from django.urls import path
from netbox.views.generic import ObjectChangeLogView

from . import models, views


urlpatterns = (
#    path("vlab_results/", views.vlab_result, name="vlab_result"),
    path("golabs/", views.golab, name="golab"),
    path("gns3srvs/", views.gns3srvListView.as_view(), name="gns3srv_list"),
    path("gns3srvs/add/", views.gns3srvEditView.as_view(), name="gns3srv_add"),
    path("gns3srvs/<int:pk>/", views.gns3srvView.as_view(), name="gns3srv"),
    path("gns3srvs/<int:pk>/edit/", views.gns3srvEditView.as_view(), name="gns3srv_edit"),
    path("gns3srvs/<int:pk>/delete/", views.gns3srvDeleteView.as_view(), name="gns3srv_delete"),
    path(
        "gns3srvs/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="gns3srv_changelog",
        kwargs={"model": models.gns3srv},
    ),
    path("ptovjobs/", views.ptovjobListView.as_view(), name="ptovjob_list"),
    path("ptovjobs/add/", views.ptovjobEditView.as_view(), name="ptovjob_add"),
    path("ptovjobs/<int:pk>/", views.ptovjobView.as_view(), name="ptovjob"),
    path("ptovjobs/<int:pk>/edit/", views.ptovjobEditView.as_view(), name="ptovjob_edit"),
    path("ptovjobs/<int:pk>/delete/", views.ptovjobDeleteView.as_view(), name="ptovjob_delete"),
    path(
        "ptovjobs/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="ptovjob_changelog",
        kwargs={"model": models.ptovjob},
    ),
    path("switchtojobs/", views.switchtojobListView.as_view(), name="switchtojob_list"),
    path("switchtojobs/add/", views.switchtojobEditView.as_view(), name="switchtojob_add"),
    path("switchtojobs/<int:pk>/", views.switchtojobView.as_view(), name="switchtojob"),
    path('switchtojobs/<int:pk>/', views.switchtojobView.as_view(), name="switchtojob"),
    path("switchtojobs/<int:pk>/edit/", views.switchtojobEditView.as_view(), name="switchtojob_edit"),
    path("switchtojobs/<int:pk>/delete/", views.switchtojobDeleteView.as_view(), name="switchtojob_delete"),
    path(
        "switchtojobs/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="switchtojob_changelog",
        kwargs={"model": models.switchtojob},
    ),
)



from netbox.filtersets import NetBoxModelFilterSet
from .models import gns3srv, ptovjob, switchtojob



class gns3srvFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = gns3srv
        fields = ['name', ]

    def search(self, queryset, name, value):
        return queryset.filter(description__icontains=value)

class ptovjobFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = ptovjob 
        fields = ['name', 'gns3prjname', 'gns3srv', 'eosuname' ]

    def search(self, queryset, name, value):
        return queryset.filter(description__icontains=value)

class switchtojobFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = switchtojob
        fields = ['name', 'switch', 'job' ]

    def search(self, queryset, name, value):
        return queryset.filter(description__icontains=value)



from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel


class gns3srv(NetBoxModel):
    name = models.CharField(max_length=100)

    class Meta:
        ordering = ("name",)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("plugins:netbox_ptov:gns3srv", args=[self.pk])

class ptovjob(NetBoxModel):
    name = models.CharField(max_length=100)

    class Meta:
        ordering = ("name",)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("plugins:netbox_ptov:ptovjob", args=[self.pk])

    gns3srv = models.ForeignKey(
        to=gns3srv,
        on_delete=models.CASCADE,
    )

    gns3prjname = models.CharField(max_length=100)

    gns3prjid = models.CharField(max_length=200)

    eosuname = models.CharField(max_length=100)

    eospasswd = models.CharField(max_length=100)

class switchtojob(NetBoxModel):
    name = models.CharField(max_length=100)
    
    class Meta:
        ordering = ("name",)
        # unique_together = ['job', 'switch']


    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("plugins:netbox_ptov:switchtojob", args=[self.pk])

    job = models.ForeignKey(
        to=ptovjob,
        on_delete=models.CASCADE,
    )

    switch = models.ForeignKey(
	to='dcim.device',
        on_delete=models.CASCADE,
    )



from dcnodatg import dcnodatg
from django.db.models import Count
from netbox.views import generic
from . import filtersets, forms, models, tables
from .models import gns3srv
from django.shortcuts import render, redirect
from django.contrib import messages
#from django.forms.models import ConfigContextModelQuerySet
import json

def golab(request):
    if request.method == 'POST':
        form = forms.golabForm(request.POST)
        if form.is_valid():
            unm = form.cleaned_data['username_in']
            pwd = form.cleaned_data['password_in']
            swl = []
            swl_in = []
            swl_in = form.cleaned_data['switchlist_multiplechoice_in']
            for swname in swl_in:
                print (swname, type(swname))        
                swl.append(str(swname))
            messages.add_message(request, messages.INFO, 'Switch-list: ' + str(swl), extra_tags='safe')
            srv = form.cleaned_data['serverselect_in'].name
            prn = form.cleaned_data['prjname_in']
            # Do something with the text (e.g., save to database)

            messages.add_message(request, messages.INFO, 'GNS3 server:' + str(srv), extra_tags='safe')
            result_out = str(dcnodatg.p_to_v(username=unm, passwd=pwd , servername=srv, switchlist=swl, prjname=prn))
            messages.add_message(request, messages.SUCCESS, 'Project Created:' + str(prn) + ' on ' + str(srv), extra_tags='safe')
            return render(request, 'golab.html', {'form': form})
    else:
        form = forms.golabForm()
        return render(request, 'golab.html', {'form': form})

class gns3srvView(generic.ObjectView):
    queryset = models.gns3srv.objects.all()
class gns3srvListView(generic.ObjectListView):
    queryset = models.gns3srv.objects.all()
    table = tables.gns3srvTable
class gns3srvEditView(generic.ObjectEditView):
    queryset = models.gns3srv.objects.all()
    form = forms.gns3srvForm
class gns3srvDeleteView(generic.ObjectDeleteView):
    queryset = models.gns3srv.objects.all()


class ptovjobView(generic.ObjectView):
    queryset = models.ptovjob.objects.all()
class ptovjobListView(generic.ObjectListView):
    queryset = models.ptovjob.objects.all()
    table = tables.ptovjobTable
class ptovjobEditView(generic.ObjectEditView):
    queryset = models.ptovjob.objects.all()
    form = forms.ptovjobForm
class ptovjobDeleteView(generic.ObjectDeleteView):
    queryset = models.ptovjob.objects.all()


class switchtojobView(generic.ObjectView):
    queryset = models.switchtojob.objects.all()
class switchtojobListView(generic.ObjectListView):
    queryset = models.switchtojob.objects.all()
    table = tables.switchtojobTable
class switchtojobEditView(generic.ObjectEditView):
    queryset = models.switchtojob.objects.all()
    form = forms.switchtojobForm
class switchtojobDeleteView(generic.ObjectDeleteView):
    queryset = models.switchtojob.objects.all()



from django.conf import settings
from netbox.plugins import PluginMenuButton, PluginMenuItem, PluginMenu


gns3srv_buttons = (
    PluginMenuButton(
        link="plugins:netbox_ptov:gns3srv_add",
        title="Add",
        icon_class="mdi mdi-plus-thick",
    ),
)

ptovjob_buttons = (
    PluginMenuButton(
        link="plugins:netbox_ptov:ptovjob_add",
        title="Add",
        icon_class="mdi mdi-plus-thick",
    ),
)


_menu_items = (
    PluginMenuItem(
        link="plugins:netbox_ptov:golab",
        link_text="Run a V-Lab",
    ),
    PluginMenuItem(
        link="plugins:netbox_ptov:gns3srv_list",
        link_text="GNS3 Servers",
        buttons=gns3srv_buttons,
    ),
#    PluginMenuItem(
#        link="plugins:netbox_ptov:ptovjob_list",
#        link_text="Virtual-lab templates",
#        buttons=ptovjob_buttons,
#    ),
)

plugin_settings = settings.PLUGINS_CONFIG.get('netbox_ptov', {})

if plugin_settings.get('top_level_menu'):
    menu = PluginMenu(  
        label="P to V-lab (Arista)",
        groups=(("P to V-lab (Arista)", _menu_items),),
        icon_class="mdi mdi-bootstrap",
    )
else:
    menu_items = _menu_items



# api/serializers.py

from rest_framework.serializers import HyperlinkedIdentityField, ValidationError
from rest_framework.relations import PrimaryKeyRelatedField

from netbox.api.fields import ChoiceField, SerializedPKRelatedField

from netbox.api.serializers import NetBoxModelSerializer
from ipam.api.serializers import IPAddressSerializer, ASNSerializer, PrefixSerializer
from tenancy.api.serializers import TenantSerializer
from dcim.api.serializers import SiteSerializer, DeviceSerializer


from netbox_ptov.models import gns3srv, ptovjob, switchtojob 

class gns3srvSerializer(NetBoxModelSerializer):
    #status = ChoiceField(choices=CommunityStatusChoices, required=False)
    #tenant = TenantSerializer(nested=True, required=False, allow_null=True)
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_ptov:gns3srv-detail")

    class Meta:
        model = gns3srv
        fields = [
            "id",
            "tags",
            "name",
        ]
        brief_fields = ("id", "name")


class ptovjobSerializer(NetBoxModelSerializer):
    #status = ChoiceField(choices=CommunityStatusChoices, required=False)
    #tenant = TenantSerializer(nested=True, required=False, allow_null=True)
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_ptov_:ptovjob-detail")

    class Meta:
        model = ptovjob
        fields = [
            "name", "eosuname", "gns3srv", "gns3prjname", "gns3prjid", "eospasswd"
        ]
        brief_fields = ("name", "gns3prjname", "gns3prjid")


class switchtojobSerializer(NetBoxModelSerializer):
    #status = ChoiceField(choices=CommunityStatusChoices, required=False)
    #tenant = TenantSerializer(nested=True, required=False, allow_null=True)
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_ptov:switchtojob-detail")

    class Meta:
        model = switchtojob
        fields = [
            "name", "switch", "job",
        ]
        brief_fields = ("name", "switch")




"""Top-level package for NetBox ptov."""

__author__ = """Mencken Davidson"""
__email__ = "mencken@gmail.com"
__version__ = "0.1.0"


from netbox.plugins import PluginConfig


class ptovConfig(PluginConfig):
    name = "netbox_ptov"
    verbose_name = "NetBox ptov plugin"
    description = "NetBox plugin for creating GNS3 virtual-labs of Arista switches."
    version = "version"
    base_url = "netbox_ptov"


config = ptovConfig



from netbox.api.routers import NetBoxRouter

from .views import (
    gns3srvViewSet, ptovjobViewSet, switchtojobViewSet, RootView
)

app_name = 'netbox_ptov'

router = NetBoxRouter()
router.APIRootView = RootView
router.register('gns3srv', gns3srvViewSet, basename='gns3srv')
router.register('ptovjob', ptovjobViewSet, basename='ptovjob')
router.register('switchtojob', switchtojobViewSet, basename='switchtojob')

urlpatterns = router.urls



from django import forms
from ipam.models import Prefix
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms.fields import CommentField, DynamicModelChoiceField
from dcim.models import devices
from .models import gns3srv, ptovjob, switchtojob


class gns3srvForm(NetBoxModelForm):
    class Meta:
        model = gns3srv 
        fields = ("name", "tags")

class ptovjobForm(NetBoxModelForm):
    class Meta:
        model = ptovjob
        fields = ("name", "gns3srv", "gns3prjname", "gns3prjid", "eosuname", "eospasswd")

class switchtojobForm(NetBoxModelForm):
    class Meta:
        model = switchtojob
        fields = ("name", "switch", "job")

class golabForm(forms.Form):
    username_in = forms.CharField(label="Enter EOS username:", widget=forms.TextInput)
    password_in = forms.CharField(label="Enter EOS password:", widget=forms.PasswordInput)
    switchlist_multiplechoice_in = forms.ModelMultipleChoiceField( queryset= devices.Device.objects.filter(device_type__manufacturer__slug="arista"), to_field_name='name')
    serverselect_in = forms.ModelChoiceField( queryset= gns3srv.objects.all(), to_field_name='name')
    prjname_in = forms.CharField(label="Enter Name for GNS3 project:", widget=forms.TextInput)




from netbox.api.viewsets import NetBoxModelViewSet
from rest_framework.routers import APIRootView

from .serializers import (
    gns3srvSerializer, ptovjobSerializer, switchtojobSerializer
)

from .. import filtersets, models

class RootView(APIRootView):
    def get_view_name(self):
        return 'ptov'

class ptovjobViewSet(NetBoxModelViewSet):
    queryset = models.ptovjob.objects.all()
    serializer_class = ptovjobSerializer
    filterset_class = filtersets.ptovjobFilterSet

class gns3srvViewSet(NetBoxModelViewSet):
    queryset = models.gns3srv.objects.all()
    serializer_class = gns3srvSerializer
    filterset_class = filtersets.gns3srvFilterSet

class switchtojobViewSet(NetBoxModelViewSet):
    queryset = models.switchtojob.objects.all()
    serializer_class = switchtojobSerializer
    filterset_class = filtersets.switchtojobFilterSet

